Ext.namespace('Zarafa.plugins.owncloudattachment.dialogs');

/**
 * @class Zarafa.plugins.owncloudattachment.dialogs.AttachmentTreePanel
 * @extends Ext.tree.TreePanel
 * Shows tree of all user files from Owncloud
 */
Zarafa.plugins.owncloudattachment.dialogs.AttachmentTreePanel = Ext.extend(Ext.tree.TreePanel, {

	/**
	 * @constructor
	 * @param {Object} config
	 */
	constructor : function(config)
	{
		config = config || {};
		Ext.applyIf(config, {
			root: {
				nodeType: 'async',
				text: '/',
				id: 'root',
				expanded : true
			},
			autoScroll: true,
			viewConfig: {
				style	:	{ overflow: 'auto', overflowX: 'hidden' }
			},
			maskDisabled: true,
			loader : new Zarafa.plugins.owncloudattachment.data.TreeLoader(),
			buttons : [
				this.createActionButtons()
			]
		});
		Zarafa.plugins.owncloudattachment.dialogs.AttachmentTreePanel.superclass.constructor.call(this, config);
	},

	/**
	 * Creates action button for adding file
	 * from Owncloud to email as attachment
	 * @return {Array}
	 * @private
	 */
	createActionButtons : function()
	{
		return [{
			xtype   : 'button',
			text    : _('&nbsp;&nbsp;Add Attachment'),
			tooltip : {
				title   : _('Add Attachment'),
				text    : _('Add selected attachment from owncloud to email attachment.')
			},
			width	: 120,
			iconCls : 'icon_owncloud',
			handler : this.downloadSelectedFilesFromOwncloudToTmp,
			scope   : this
		}];
	},

	/**
	 * Sends a query with ids of selected files from user's Owncloud folder
	 * @private
	 */
	downloadSelectedFilesFromOwncloudToTmp : function()
	{
		var selectedNodes =  this.dialog.treePanel.getChecked();
		var idsList = [];
		var emailRecord = this.dialog.record;
		var attachmentStore = emailRecord.getAttachmentStore();

		Ext.each(selectedNodes, function(node, index) {
			idsList.push( node.id );
		});
		
		if(idsList.length < 1) {
			Ext.MessageBox.show({
				title   : _('Warning'),
				msg     : _('You have to choose at least one file!'),
				icon    : Ext.MessageBox.WARNING,
				buttons : Ext.MessageBox.OK
			});
		} else {
			try {
				this.dialog.treePanel.disable();
				container.getRequest().singleRequest(
					'owncloudmodule',
					'download-to-tmp',
					{
						ids : idsList,
						dialog_attachments: attachmentStore.getId(),
						uid: container.getSettingsModel().get('zarafa/v1/plugins/owncloudattachment/username'),
						pass: container.getSettingsModel().get('zarafa/v1/plugins/owncloudattachment/password'),
						oc_server: container.getSettingsModel().get('zarafa/v1/plugins/owncloudattachment/owncloud_server'),
						oc_base: container.getSettingsModel().get('zarafa/v1/plugins/owncloudattachment/owncloud_path'),
						oc_ssl: container.getSettingsModel().get('zarafa/v1/plugins/owncloudattachment/use_ssl')
					},
					new Zarafa.plugins.owncloudattachment.data.ResponseHandler({
						successCallback : this.addDownloadedFilesAsAttachmentToEmail.createDelegate(this)
					})
				);
			} catch (e) {
				Ext.MessageBox.show({
					title   : _('Warning'),
					msg     : e.getMessage(),
					icon    : Ext.MessageBox.WARNING,
					buttons : Ext.MessageBox.OK
				});
			}
		}
	},

	/**
	 * Converts received file information to attachment record
	 * @param {Object} downloadedFileInfo
	 * @private
	 */
	convertDownloadedFileInfoToAttachmentRecord : function(downloadedFileInfo)
	{
		var attachmentRecord = Zarafa.core.data.RecordFactory.createRecordObjectByObjectType(Zarafa.core.mapi.ObjectType.MAPI_ATTACH);

		attachmentRecord.set('tmpname', downloadedFileInfo.tmpname);
		attachmentRecord.set('name', downloadedFileInfo.name);
		attachmentRecord.set('size', downloadedFileInfo.size);
		return attachmentRecord;
	},

	/**
	 * Adds files received from the user's Owncloud folder to
	 * the email as attachments
	 * @param {Object} downloadedFilesInfoArray
	 * @private
	 */
	addDownloadedFilesAsAttachmentToEmail : function(downloadedFilesInfoArray)
	{
		var emailRecord = this.dialog.record;
		var attachmentStore = emailRecord.getAttachmentStore();

		var attachmentRecord = null;
		Ext.each(downloadedFilesInfoArray, function(downloadedFileInfo, index) {
			attachmentRecord = this.convertDownloadedFileInfoToAttachmentRecord(downloadedFileInfo);
			attachmentStore.add(attachmentRecord);
		}, this);
		this.dialog.close();
	}
});

Ext.reg('owncloudattachment.treepanel', Zarafa.plugins.owncloudattachment.dialogs.AttachmentTreePanel);
